module Ohm
  class McCree
    def initialize
      @mutex = Mutex.new
      @redis = Redic.new
      @db =  get_db(Ohm.redis.url) || '0'
      enable_expired_event_notifications
    end

    def get_db redis_url
      /^redis:\/\/.*:\d+\/(?<db>\d+)$/.match(redis_url)&.[](:db)
    end

    def overwatching
      @thread = Thread.new { overwatch } 
    rescue
      self.class.new.overwatching
    end

    def overwatch
      @mutex.synchronize do
        subscribe { |message, event, key| Ohm::Model.it_is_high_noon(key) if key }
      end
    end

    def subscribe
      @redis.call("SUBSCRIBE", "__keyevent@#{@db}__:expired")
      loop { yield @redis.client.read }
    end

    def enable_expired_event_notifications
      config_key = 'notify-keyspace-events'
      @nke_was = @redis.call(
        'config', 'get', config_key
      )&.last
      nke = ( @nke_was.chars | ['E', 'x'] ).join
      @redis.call(
        'config', 'set', config_key, nke
      )
    end

    def disable_expired_event_notifications
      @redis.call('config', 'set', 'notify-keyspace-events', @nke_was)
    end
  end
end
